using System;
using System.Linq;
using System.Reflection;
using ImageProcessingGraph.Editor.Nodes.Types.Utils.Connector;
using ImageProcessingGraph.Editor.Unity_Image_Processing.Scripts.Editor.Windows;
using UnityEditor.Experimental.GraphView;
using UnityEngine;
using UnityEngine.UIElements;

namespace ImageProcessingGraph.Editor
{
    public class IPTPort : Port
    {
        public FieldInfo fieldInfo;
        private VisualElement _exposedPropertyContainer;

        public delegate void OnPortConnectedEvent();
        public OnPortConnectedEvent OnPortConnected;
        
        public delegate void OnPortDisconnectedEvent();
        public OnPortDisconnectedEvent OnPortDisconnected;
        
        public VisualElement ExposedPropertyContainer
        {
            set
            {
                _exposedPropertyContainer = value;
            }
            get
            {
                return _exposedPropertyContainer;
            }
        }
        
        protected IPTPort(Orientation portOrientation, Direction portDirection, Capacity portCapacity, Type type) : base(portOrientation, portDirection, portCapacity, type)
        {
            
        }

        public override void Connect(Edge edge)
        {
            ImageProcessingGraphNodeVisual IPTVis = this.node as ImageProcessingGraphNodeVisual;

            IPTPort extenalPort = null;
            
            if (IPTVis.GraphNode is GenericConnection bleh)
            {
                foreach (var port in IPTVis.AllPorts)
                {
                    port.portType = edge.output.portType;
                }
            }
            
            base.Connect(edge);
            OnPortConnected?.Invoke();
        }

        public override void Disconnect(Edge edge)
        {
            ImageProcessingGraphNodeVisual IPTVis = this.node as ImageProcessingGraphNodeVisual;

            if (IPTVis.GraphNode is GenericConnection bleh)
            {
                bool allPortsDisconnected = true;

                foreach (var port in IPTVis.AllPorts)
                {
                    if (port.connections != null && port.connections.Count() > 0)
                    {
                        allPortsDisconnected = false;
                        break;
                    }
                }

                if (allPortsDisconnected)
                {
                    foreach (var port in IPTVis.AllPorts)
                    {
                        port.portType = typeof(object);
                    }

                    bleh.InternalType = null;
                }
            }

            
            base.Disconnect(edge);
            OnPortDisconnected?.Invoke();
        }


        private void PublicOnConnected(Port obj)
        {
            throw new NotImplementedException();
        }

        public static IPTPort Create(IEdgeConnectorListener connectorListener, bool isInput, Type type)
        {
            var port = new IPTPort(Orientation.Horizontal, isInput ? Direction.Input : Direction.Output,
                isInput ? Capacity.Single : Capacity.Multi, type)
            {
                m_EdgeConnector = new EdgeConnector<Edge>(connectorListener),
            };
            port.AddManipulator(port.m_EdgeConnector);
            return port;
        }

    }
}